import simulation.events.*;
import simset.*;
import random.*;

public class CarWashSimulation extends Simulation { 
    int noOfCarWashers;
    double simPeriod = 200;
    Head tearoom = new Head();
    Head waitingLine = new Head();
    int noOfCustomers, maxLength;
    double throughTime;
    Random random = new Random(5);
    long startTime = System.currentTimeMillis();

    CarWashSimulation(int n) {  
        noOfCarWashers = n; 
        for (int i = 1; i <= noOfCarWashers; i++)
            new CarWasher().into(tearoom);
        new CarArrival().schedule(0);
        runSimulation(simPeriod + 1000000);
        report();
    }
    
    void report() {
        System.out.println(noOfCarWashers + " car washer simulation");
        System.out.println("No.of cars through the system = " + 
                           noOfCustomers); 
        java.text.NumberFormat fmt = java.text.NumberFormat.getNumberInstance();
        fmt.setMaximumFractionDigits(2);
        System.out.println("Av.elapsed time = " + 
                           fmt.format(throughTime/noOfCustomers));
        System.out.println("Maximum queue length = " + maxLength);
        System.out.println("\nExecution time: " +
                           fmt.format((System.currentTimeMillis() 
                                       - startTime)/1000.0) +			   
                           " secs.\n");
    }

    class CarWasher extends Link {}

    class Car extends Link {
        double entryTime = time();
    }

    class CarArrival extends TimeEvent {
        public void actions() {
            if (time() <= simPeriod) {
	              Car theCar = new Car();
	              theCar.into(waitingLine);
	              int qLength = waitingLine.cardinal();
	              if (maxLength < qLength) 
	                  maxLength = qLength;
	              new StartCarWashing(theCar).schedule();
	              schedule(time() +  random.negexp(1/11.0));
	          }
        }	
    }

    class StopCarWashing extends TimeEvent {
        CarWasher theCarWasher;
        Car theCar;
	
        StopCarWashing(CarWasher cw, Car c)  
          { theCarWasher = cw; theCar = c; }
	
        public void actions() { 
            noOfCustomers++;
            throughTime += time() - theCar.entryTime; 
            theCarWasher.into(tearoom);
        }
    }

    class StartCarWashing extends StateEvent {
        Car theCar;
	
        StartCarWashing(Car c) { theCar = c; }
	
        public boolean condition() { 
            return theCar == (Car) waitingLine.first() && !tearoom.empty();	
        }
	
        public void actions() { 
            theCar.out();
            CarWasher theCarWasher = (CarWasher) tearoom.first();
            theCarWasher.out();
            new StopCarWashing(theCarWasher, theCar).schedule(time() + 10);
        }
    }
	
    public static void main(String args[]) {
        new CarWashSimulation(1);
        new CarWashSimulation(2);
    }
}