package javaCoroutine;

public abstract class Coroutine extends Thread { 
	final public void run() {
		body();
        if (!terminated) {
		    terminated = true;
			detach();
        }
	}

	abstract public void body();
	
	private static void error(String msg) {
        throw new RuntimeException(msg);
    }
			
    public static final void resume(Coroutine next) {
         if (next == null) 
             error("resume non-existing coroutine");
         if (next.terminated)
             error("resume terminated coroutine");
         if (next.caller != null)
             error("resume attached coroutine");
         if (next == current) 
             return; 
         while (next.callee != null)
              next = next.callee;
         next.enter(); 		
    }
	
    public static final void call(Coroutine next) {
        if (next == null) 
            error("call non-existing coroutine");
        if (next.terminated)
            error("call terminated coroutine");
        if (next.caller != null)
            error("call attached coroutine");
        if (current != null)
            current.callee = next;
        next.caller = current;
        while (next.callee != null)
            next = next.callee;
        if (next == current) 
            error("call operating coroutine");
        next.enter();	 
    }
	
	public static void detach() {
		Coroutine next = current.caller;
		if (next != null) {
			current.caller = next.callee = null;
			next.enter();
		}
		else if (main != null && current != main)
			main.enter();
	}
	
	private Coroutine caller, callee;
	protected boolean terminated;
	private static Coroutine current, main;

	private void enter() {
		if (current == null) {
            main = current = this;
			start();
			return;
		}
		Coroutine old_current = current;
        current = this;
        if (!isAlive()) 
            start(); 
	    else 
            interrupt(); 
	    if (old_current.terminated)
            return;
        synchronized(old_current) {
			try {
                old_current.wait();
			} catch(InterruptedException e) {}
		}
	}	
}