package javaCoroutine;

public abstract class Coroutine implements Runnable {
    final public void run() {
        body();
        if (!terminated) {
            terminated = true;
            detach();
        }
        myThread = null;
        if (main == this)  
            main = current = null;
    }
	
    protected abstract void body();
			
    private static void error(String msg) {
        throw new RuntimeException(msg);
    }
			
    public static final void resume(Coroutine next) {
         if (next == null) 
             error("resume non-existing coroutine");
         if (next.terminated)
             error("resume terminated coroutine");
         if (next.caller != null)
             error("resume attached coroutine");
         if (next == current) 
             return; 
         while (next.callee != null)
              next = next.callee;
         next.enter(); 		
    }
	
    public static final void call(Coroutine next) {
        if (next == null) 
            error("call non-existing coroutine");
        if (next.terminated)
            error("call terminated coroutine");
        if (next.caller != null)
            error("call attached coroutine");
        if (current != null)
            current.callee = next;
        next.caller = current;
        while (next.callee != null)
            next = next.callee;
        if (next == current) 
            error("call operating coroutine");
        next.enter();	 
    }
	
    public static final void detach() {
        Coroutine next = current.caller;
        if (next != null) {
            current.caller = next.callee = null;
            next.enter();
        }
        else if (main != null && current != main)
            main.enter();
    }

    public static final Coroutine currentCoroutine() {
        return current;
    }

    public static final Coroutine mainCoroutine() {
        return main;
    }
	
    private Coroutine caller, callee;
    private static Coroutine current, main;
    protected boolean terminated;
    private Thread myThread;
	
    private void enter() {
        if (myThread == null) 
            myThread = new Thread(this);
        if (current == null) { 
            current = main = this;
            myThread.start();
            return;
        }
        Coroutine old_current = current;
        synchronized(old_current) {
            synchronized(this) { 
                current = this;
                if (!myThread.isAlive())
                    myThread.start();
                else
                    notify(); 
            }
            if (old_current.terminated)
                return;
            try {
                old_current.wait();
            } catch(InterruptedException e) {}
        }
    }	
}