/*
  File: StateEvent.java

  Originally written by Keld Helsgaun and released into the public domain.
  This may be used for any purposes whatsoever without acknowledgment.

  History:
  Date       Who                What
  5May2000   kh         Created public version
*/


package simulation.events;

/**
* This class may used to describe a state event, i.e., an event 
* scheduled to occur to when the state of the system 
* fulfills a specified condition.
* <p>
* State events are created as instances of <tt>StateEvent</tt>
* -derived classes that override the abstract methods <tt>actions</tt>
* and <tt>condition</tt>.
* <p>
* The <tt>actions</tt> method is used to specify the actions
* of a class of state events. The actions of a state event are
* executed when its <tt>Condition</tt> returns true.
*
* @see simulation.events.Event
* @see simulation.events.TimeEvent
* @see simulation.events.Simulation  
*/
public abstract class StateEvent extends Event {
    /**
    * The state condition for this event.
    */
    protected abstract boolean condition();
    
    /**
    * Schedules this state event. 
    * Its time of occurence depends on its <tt>condition</tt>.
    * <p>
    * The activity is inserted at the back of the list of 
    * waiting activities.
    */  
    public final void schedule() {
        cancel();
        suc = waitList.suc;
        suc.pred = waitList.suc = this;
        pred = waitList;
    }	 
}